//--------------------------------------------------------------------------------
//
// Checkers     
// ========
//
// by:- David A.S.Crofts
//
// 23 Brisbane Street 
// Berwick Victoria 3806
// Australia
//
// dasc1961@netscape.net
//
// http://www.davidcrofts.com.au/
//
//--------------------------------------------------------------------------------

#include "checkers.h"

#include <qapplication.h>
#include <qpainter.h>
#include <qdrawutil.h>
#include <qcheckbox.h>
#include <qlabel.h>
#include <qlayout.h>
#include <stdlib.h>
#include <qdatetime.h>
#include <qstring.h>
#include <qtimer.h>
#include <qmessagebox.h>
#include <qfile.h>

//--------------------------------------------------------------------------------
//  External Variables and Constants
//--------------------------------------------------------------------------------

extern unsigned char num_moves[4][32];
extern unsigned char moves[4][32][4];
extern unsigned char jumps[4][32][4];

extern CheckersPieceBlock computer_turn_nos;
extern CheckersPieceBlock opponent_turn_nos;

extern CheckersPieceBlock *t;

extern bool jump_found;
extern int player_jump_found;

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

CheckersPieceBlock::CheckersPieceBlock (unsigned char mp, unsigned char mk,
					unsigned char op, unsigned char ok)
{
    mover_piece = mp;
    mover_king = mk;
    opponent_piece = op;
    opponent_king = ok;
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

void
CheckersState::copy_state (CheckersState * q)
{
    int i;

    for (i = 0; i < 32; ++i)
	q->square[i] = square[i];

    q->turn = turn;
    q->piece = piece;

    q->jump_start_square = jump_start_square;
    q->jump_found = jump_found;
    q->val = val;
    q->tdepth = tdepth;
    q->game_over = game_over;
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

void
CheckersState::copy_state_play (CheckersState * q)
{
    int i;

    for (i = 0; i < 32; ++i)
	q->square[i] = square[i];

    q->turn = turn;
    q->piece = piece;
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

bool CheckersState::states_equal (CheckersState * q)
{
    int
	i;

    for (i = 0; i < 32; ++i)
	if (square[i] != q->square[i])
	    break;

    return (i == 32);
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

int
CheckersState::value ()
{
    int i, x1, x2, x3, x4;

    if (square[0] == PlayerPiece && square[2] == PlayerPiece)
	x1 = 2;
    else
	x1 = 0;

    if (square[29] == ComputerPiece && square[31] == ComputerPiece)
	x3 = 2;
    else
	x3 = 0;

    x2 = x4 = 0;

    for (i = 0; i < 4; ++i)
	switch (square[i]) {
	case PlayerPiece:
	    x1 += 8;
	    break;
	case PlayerKing:
	    x2 += 11;
	    break;
	case ComputerKing:
	    x4 += 11;
	}

    for (i = 4; i < 28; ++i)
	switch (square[i]) {
	case PlayerPiece:
	    x1 += 8;
	    break;
	case PlayerKing:
	    x2 += 12;
	    break;
	case ComputerPiece:
	    x3 += 8;
	    break;
	case ComputerKing:
	    x4 += 12;
	}

    for (i = 28; i < 32; ++i)
	switch (square[i]) {
	case PlayerKing:
	    x2 += 11;
	    break;
	case ComputerPiece:
	    x3 += 8;
	    break;
	case ComputerKing:
	    x4 += 11;
	}

    if (!x1 && !x2) {

	if (turn == PlayerPiece)
	    return (-25000 - x3 - x4);
	else
	    return (25000 + x3 + x4);

    }
    else if (!x3 && !x4) {

	if (turn == PlayerPiece)
	    return (25000 + x1 + x2);
	else
	    return (-25000 - x1 - x2);

    }
    else if (turn == PlayerPiece)

	return (x1 + x2 - x3 - x4);

    else

	return (x3 + x4 - x1 - x2);
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

void
CheckersState::RemoveStateFlags ()
{

    int i;

    for (i = 0; i < 32; ++i) {
	if (square[i] > HighestValidPiece) {
	    switch (square[i]) {

	    case HiLightedPlayerPiece:

		square[i] = PlayerPiece;
		break;

	    case HiLightedPlayerKing:

		square[i] = PlayerKing;
		break;

	    case HiLightedBlankSquare:
	    case HiLightedEndSquare:
	    case HiLightedMidSquare:
	    case ComputerJumpedSquare:
	    case ComputerStartSquare:

		square[i] = BlankSquare;
		break;

	    case ComputerEndSquarePiece:

		square[i] = ComputerPiece;
		break;

	    case ComputerEndSquareKing:

		square[i] = ComputerKing;
		break;
	    }
	}
    }
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

void
CheckersState::set_clump_value ()
{
    int i, j, row1, col1, row2, col2, row_dist, col_dist;

    clump = 0;

    for (i = 0; i < 32; ++i) {

	if (square[i] == ComputerPiece || square[i] == ComputerKing) {

	    row1 = i / 4;
	    col1 = (i % 4) * 2 + 1 - row1 + (row1 / 2) * 2;

	    for (j = 0; j < 32; ++j) {

		if (square[j] == PlayerPiece || square[j] == PlayerKing) {

		    row2 = j / 4;
		    col2 = (j % 4) * 2 + 1 - row2 + (row2 / 2) * 2;

		    if (square[i] == ComputerPiece)

			clump -= (i / 4) * 16;	//First priority to king piece

		    else {

			if ((row_dist = (row1 - row2)) < 0)
			    row_dist = -row_dist;
			if ((col_dist = (col1 - col2)) < 0)
			    col_dist = -col_dist;

			clump += 16 - row_dist - col_dist;	//Kings chase main clump of oponents
		    }
		}
	    }
	}
    }
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

CheckersSucessor::CheckersSucessor ()
{
    num = 0;
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

void
CheckersSucessor::set (CheckersState * p)
{
    CheckersState *new_state;
    unsigned int c, i, j, k, l;

    num = 0;
    if (p->turn == ComputerPiece)
	t = &computer_turn_nos;
    else
	t = &opponent_turn_nos;
    jump_found = false;
    for (i = 0; i < 32; ++i) {
	c = p->square[i];
	if (c == t->mover_piece || c == t->mover_king) {
	    p->piece = i;
	    add_jumps (p, false);
	    if (!jump_found) {
		j = num_moves[c][i];
		for (k = 0; k < j; ++k) {
		    l = moves[c][i][k];
		    if (p->square[--l] == BlankSquare) {
			new_state = new CheckersState;
			p->copy_state_play (new_state);
			new_state->square[i] = BlankSquare;
			if (l < 4 || l > 27)
			    new_state->square[l] = t->mover_king;
			else
			    new_state->square[l] = c;
			new_state->turn = t->opponent_piece;
			ptr[num] = new_state;
			++num;
		    }
		}
	    }
	}
    }
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

void
CheckersSucessor::set_single_step (CheckersState * p)
{
    CheckersSucessor *q;
    CheckersSucessor *r;
    CheckersState *new_state;
    unsigned int c, i, j, k, l;

    q = NULL;
    p->turn = PlayerPiece;
    t = &opponent_turn_nos;	// Needed for consider_jumps()

    if ((i = p->piece) > 127) {
	i -= 128;
    }
    c = p->square[i];
    if (c == PlayerPiece || c == PlayerKing) {
	if ((q = consider_jumps (p)) != NULL) {
	    for (j = 0; j < q->num; ++j) {
		if (q->ptr[j]->piece < 0) {	// Piece has Kinged through jumping
		    q->ptr[j]->piece += 64;	// Set correct square, mark as END
		    q->ptr[j]->turn = ComputerPiece;
		}
		else if ((r = consider_jumps (q->ptr[j])) != NULL) {
		    q->ptr[j]->piece += 128;	// mark as MID Square
		    r->free ();
		}
		else {
		    q->ptr[j]->turn = ComputerPiece;
		}
	    }
	}
	else if (!player_jump_found) {	// Global Variable
	    j = num_moves[c][i];
	    for (k = 0; k < j; ++k) {
		l = moves[c][i][k] - 1;
		if (p->square[l] == BlankSquare) {
		    new_state = new CheckersState;
		    p->copy_state_play (new_state);
		    new_state->square[i] = BlankSquare;
		    if (l < 4 || l > 27)
			new_state->square[l] = PlayerKing;
		    else
			new_state->square[l] = c;
		    new_state->piece = l;	// Mark as END Square
		    new_state->turn = ComputerPiece;
		    if (q == NULL)
			q = new CheckersSucessor;
		    q->ptr[q->num] = new_state;
		    ++(q->num);
		}
	    }
	}
    }
    if (q != NULL) {
	for (i = 0; i < q->num; ++i)
	    ptr[i] = q->ptr[i];
	num = q->num;
	delete q;
    }
    else
	num = 0;
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

void
CheckersSucessor::add_jumps (CheckersState * p, bool jumped)
{
    CheckersSucessor *r;
    unsigned int i;

    r = consider_jumps (p);
    if (r == NULL) {
	if (jumped) {
	    if (!jump_found) {
		for (i = 0; i < num; ++i)
		    delete ptr[i];
		num = 0;
		jump_found = true;
	    }
	    p->turn = t->opponent_piece;
	    ptr[num] = p;
	    ++num;
	}
    }
    else {
	for (i = 0; i < r->num; ++i)
	    add_jumps (r->ptr[i], true);
	delete r;
	if (jumped) {
	    delete p;
	}
    }
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

CheckersSucessor *
CheckersSucessor::consider_jumps (CheckersState * p)
{
    CheckersSucessor *q;
    CheckersState *new_state;
    int i;
    unsigned int j, k, l, m, n, o;

    q = NULL;
    i = p->piece;
    if (i < 0)
	return (q);
    j = p->square[i];
    k = num_moves[j][i];
    for (l = 0; l < k; ++l) {
	m = moves[j][i][l];
	n = p->square[--m];
	if (n == t->opponent_piece || n == t->opponent_king) {
	    o = jumps[j][i][l];
	    if (o != 0 && p->square[--o] == BlankSquare) {
		new_state = new CheckersState;
		p->copy_state_play (new_state);
		new_state->square[i] = BlankSquare;
		new_state->square[m] = BlankSquare;
		if (o < 4 || o > 27) {
		    new_state->square[o] = t->mover_king;
		    if (j == t->mover_piece)
			new_state->piece = o - 64;
		    else
			new_state->piece = o;
		}
		else {
		    new_state->square[o] = j;
		    new_state->piece = o;
		}
		if (q == NULL)
		    q = new CheckersSucessor;
		q->ptr[q->num] = new_state;
		++(q->num);
	    }
	}
    }
    return (q);
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

void
CheckersSucessor::end_game_tweek ()
{
    long clump_max;
    int i, j;

    if (num < 2)
	return;

    for (i = 0; i < (signed) num; ++i)
	ptr[i]->set_clump_value ();

    clump_max = ptr[0]->clump;

    for (i = 1; i < (signed) num; ++i)
	if ((j = ptr[i]->clump) > clump_max)
	    clump_max = j;

    for (i = 0, j = 0; i < (signed) num; ++i) {
	if (ptr[i]->clump < clump_max)
	    delete ptr[i];
	else {
	    if (j < i)
		ptr[j] = ptr[i];
	    ++j;
	}
    }

    num = j;
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

void
CheckersSucessor::free ()
{
    unsigned int i;

    for (i = 0; i < num; ++i)
	delete ptr[i];

    delete this;
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

CheckersAlternatives::CheckersAlternatives (CheckersState * p,
					    unsigned int depth,
					    QWidget * parent,
					    const char *name)
    :
QWidget (parent, name)
{
    CheckersSucessor *q;
    int max, t;
    unsigned int i, j;

    q = new CheckersSucessor;
    q->set (p);
    setAlternatives (new CheckersSucessor);
    max = -30000;
    for (i = 0; i < q->num; ++i) {
	t = -f (q->ptr[i], -30000, 30000, depth - 1);
	if (t == max) {
	    j = alternative_moves ()->num;
	    alternative_moves ()->ptr[j] = new CheckersState;
	    q->ptr[i]->copy_state_play (alternative_moves ()->ptr[j]);
	    alternative_moves ()->ptr[j]->piece = -max;
	    ++(alternative_moves ()->num);
	}
	else if (t > max) {
	    max = t;
	    for (j = 0; j < alternative_moves ()->num; ++j) {
		delete alternative_moves ()->ptr[j];
	    }
	    alternative_moves ()->num = 1;
	    alternative_moves ()->ptr[0] = new CheckersState;
	    q->ptr[i]->copy_state_play (alternative_moves ()->ptr[0]);
	    alternative_moves ()->ptr[0]->piece = -max;
	}
    }
    q->free ();
    alternative_moves ()->end_game_tweek ();
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

int
CheckersAlternatives::f (CheckersState * p, int alpha, int beta,
			 unsigned int depth)
{
    CheckersSucessor *q;
    int t;
    unsigned int i;

    if (depth == 0)
	return (p->value ());

    q = new CheckersSucessor;
    q->set (p);

    if (q->num == 0 && (alpha = p->value ()) > -20000 && alpha < 20000) {

	if (!StalemateLegal)
	    alpha -= 25000;
	else if (alpha < -16)
	    alpha += 16;
	else if (alpha > 16)
	    alpha -= 16;

    }

    for (i = 0; i < q->num && alpha < beta; ++i) {

	t = -f (q->ptr[i], -beta, -alpha, depth - 1);

	if (t > alpha)
	    alpha = t;

    }

    q->free ();
    return (alpha);
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

CheckersButton::CheckersButton (QWidget * parent)
:
QPushButton (parent)
{
    setMouseTracking (true);
    setPos (32);
    setTypeNoPaint (GreenSquare);
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

void
CheckersButton::drawButtonLabel (QPainter * p)
{
    QColor red (QColor (255, 255, 224));
    QColor green (QColor (119, 162, 109));
    QColor white (QColor (200, 195, 101));
    QColor yellow (QColor (255, 255, 0));
    QColor blue (QColor (96, 160, 192));
    QColor light_green (QColor (0, 255, 0));
    QColor purple (QColor (255, 216, 240));

    int pad;

    QRect r = rect ();

    switch (t) {
    case GreenSquare:
	p->setPen (green);
	p->setBrush (green);
	break;
    case HiLightedEndSquare:
	p->setPen (yellow);
	p->setBrush (yellow);
	break;
    case HiLightedMidSquare:
	p->setPen (light_green);
	p->setBrush (light_green);
	break;
    case HiLightedPlayerPiece:
    case HiLightedPlayerKing:
    case HiLightedBlankSquare:
    case ComputerStartSquare:
    case ComputerEndSquarePiece:
    case ComputerEndSquareKing:
	p->setPen (blue);
	p->setBrush (blue);
	break;
    case ComputerJumpedSquare:
	p->setPen (purple);
	p->setBrush (purple);
	break;
    default:
	p->setPen (white);
	p->setBrush (white);
	break;
    }

    p->drawRect (r);

    pad = (r.width () + r.height ()) / 20;

    if (t == ComputerPiece || t == ComputerKing || t == ComputerEndSquarePiece
	|| t == ComputerEndSquareKing) {

	p->setPen (black);
	p->setBrush (black);

	p->drawPie (r.left () + pad, r.top () + pad, r.width () - 2 * pad,
		    r.height () - 2 * pad, 0, 5760);
    }

    if (t == PlayerPiece || t == PlayerKing || t == HiLightedPlayerPiece
	|| t == HiLightedPlayerKing) {

	p->setPen (red);
	p->setBrush (red);

	p->drawPie (r.left () + pad, r.top () + pad, r.width () - 2 * pad,
		    r.height () - 2 * pad, 0, 5760);

	p->setPen (black);

	p->drawArc (r.left () + pad, r.top () + pad, r.width () - 2 * pad,
		    r.height () - 2 * pad, 0, 5760);
    }

    if (t == PlayerKing || t == ComputerKing || t == HiLightedPlayerKing
	|| t == ComputerEndSquareKing) {

	if (t == PlayerKing || t == HiLightedPlayerKing)
	    p->setPen (black);
	else
	    p->setPen (red);
	p->drawLine (r.topLeft () +
		     QPoint (r.width () / 2, r.height () / 4 + 2),
		     r.topLeft () + QPoint (r.width () / 2,
					    r.height () * 3 / 4 - 1));
	p->drawLine (r.topLeft () +
		     QPoint (r.width () / 4 + 2, r.height () / 2),
		     r.topLeft () + QPoint (r.width () * 3 / 4 - 1,
					    r.height () / 2));
    }

}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

void
CheckersButton::enterEvent (QEvent *)
{
    setMouseMoves (0);
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

void
CheckersButton::leaveEvent (QEvent *)
{
    if (t == HiLightedPlayerPiece || t == HiLightedPlayerKing)

	emit mouse (p, false);
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

void
CheckersButton::mouseMoveEvent (QMouseEvent *)
{
    if ((t == PlayerPiece || t == PlayerKing)) {
	setMouseMoves (mouse_moves () + 1);
	if (mouse_moves () == 4)
	    emit mouse (p, true);
    }
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

CheckersGameButton::CheckersGameButton (const QString & text,
					QWidget * parent, const char *name)
    :
QPushButton (text, parent, name)
{
    setMouseTracking (true);
    setMouseOver (false);
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

void
CheckersGameButton::drawButtonLabel (QPainter * p)
{
    QColor red (QColor (255, 255, 224));

    if (text () == "I Win !!!") {
	p->setPen (QColor (255, 0, 0));
	p->setBrush (QColor (255, 0, 0));
    }
    else if (text () == "You Win !!!") {
	p->setPen (QColor (0, 255, 0));
	p->setBrush (QColor (0, 255, 0));
    }
    else {
	p->setPen (red);
	p->setBrush (red);
    }

    if (mouse_over ()) {
	p->setPen (black);
	p->setBrush (black);
    }

    p->drawRect (rect ());

    if (mouse_over ())
	p->setPen (red);
    else
	p->setPen (black);

    p->drawText (rect (), Qt::AlignCenter, text ());
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

void
CheckersGameButton::enterEvent (QEvent *)
{
    setMouseOver (true);
    repaint ();
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

void
CheckersGameButton::leaveEvent (QEvent *)
{
    setMouseOver (false);
    repaint ();
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

CheckersGrid::CheckersGrid (QWidget * parent, const char *name)
    :
QWidget (parent, name)
{
    int i, j, k;

    setCheckersButtons (new CheckersButtons (64));	// create real buttons

    QGridLayout *g = new QGridLayout (this, 8, 8, 1);
    for (k = 0; k < 64; k += 8)	// create and connect buttons
	for (j = 0; j < 8; j++) {
	    i = (56 - k) + j;

	    CheckersButton *cb = new CheckersButton (this);
	    if ((!((i / 8) % 2) && !(i % 2)) || (((i / 8) % 2) && (i % 2))) {
		cb->setEnabled (TRUE);
	    }
	    else {
		cb->setEnabled (FALSE);
	    }
	    connect (cb, SIGNAL (pressed ()), parent,
		     SLOT (buttonClicked ()));
	    connect (cb, SIGNAL (mouse (int, bool)), parent,
		     SLOT (moveMouse (int, bool)));
	    g->addWidget (cb, k / 8, j % 8);
	    checkers_buttons ()->insert (i, cb);
	}

}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

CheckersLabel::CheckersLabel (QWidget * parent, const char *name, WFlags f)
    :
QLabel (parent, name, f)
{

}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

CheckersGameBoard::CheckersGameBoard (QWidget * parent, const char *name)
    :
QWidget (parent, name)
{
    int initDepth = 0;

    setMinimumWidth ((BUTTON_HEIGHT + 1) * 10);
    setMinimumHeight ((BUTTON_HEIGHT + 1) * 10);

    setPalette (QColor (255, 255, 224));

    showing = false;

    r = NULL;
    HeadPrevStates = NULL;
    HeadNextStates = NULL;
    player_jump_found = false;

    compulsory_jump_found = false;
    compulsory_jump_start_square = 32;

    QFile f (".checkers");

    if (f.exists ()) {
	f.open (IO_ReadOnly);
	int len = f.size ();
	char *buf = new char[len + 8];
	f.readLine (buf, len);
	sscanf (buf, "%d", &initDepth);
	delete buf;
	f.close ();
    }

    if (initDepth < 4 || initDepth > 16)
	initDepth = 12;

    setDepth (initDepth);
    setTempDepth (initDepth);
    set_mouse_square (32);
    setInverted (false);

    QVBoxLayout *v = new QVBoxLayout (this, 0);
    QHBoxLayout *h = new QHBoxLayout;
    grid = new CheckersGrid (this);
    buttons = grid->checkers_buttons ();

    v->addWidget (new CheckersLabel (this));
    v->addLayout (h);
    h->addWidget (new CheckersLabel (this));
    h->addWidget (grid);
    grid->setPalette (black);
    h->addWidget (new CheckersLabel (this));
    v->addWidget (new CheckersLabel (this));

    QTime t = QTime::currentTime ();
    srand (t.hour () * 12 + t.minute () * 60 + t.second () * 60);

    newGameButton (true);

    updateButtons ();

    MoveTimer = startTimer (50);
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

CheckersGameBoard::~CheckersGameBoard ()
{
    CheckersState *p;

    while (HeadPrevStates != NULL) {
	p = HeadPrevStates;
	HeadPrevStates = HeadPrevStates->memPtr;
	delete (p);
    }

    while (HeadNextStates != NULL) {
	p = HeadNextStates;
	HeadNextStates = HeadNextStates->memPtr;
	delete (p);
    }

    if (r != NULL)
	r->free ();

    delete buttons;
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

void
CheckersGameBoard::makeMove ()
{
    setState (Thinking);
    emit moving ();
    MoveTimer = startTimer (50);	// Won't display "*** MOVING ***" otherwize
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

void
CheckersGameBoard::showMoveButton ()
{
    CheckersState *p;

    p = new CheckersState;

    new_move.copy_state (p);
    p->RemoveStateFlags ();

    if (p->states_equal (&old_move)) {

	emit showed ();

    }
    else {

	showing = true;
	old_move.copy_state (&new_move);
	updateButtons ();
	MoveTimer = startTimer (1000);
    }

    delete p;
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

void
CheckersGameBoard::timerEvent (QTimerEvent * e)
{
    if (e->timerId () == MoveTimer) {
	killTimer (MoveTimer);

	if (state () == Thinking) {

	    computerMove ();

	    emit moved ();

	}
	else if (state () == Undoing || state () == Redoing) {

	    updateButtons ();

	    emit moved ();

	}
	else if (showing) {

	    move.copy_state (&new_move);

	    setDefaultPiece (BlankSquare);
	    player_jump_found = new_move.jump_found;
	    setValue (new_move.val);
	    if (new_move.jump_start_square != 32) {
		setBoardOptions (new_move.jump_start_square);	// Resets Global Variable r
		compulsory_jump_found = true;
		MouseOverDisabled = true;
	    }
	    else {
		compulsory_jump_found = false;
		if (new_move.game_over)
		    MouseOverDisabled = true;
		else
		    MouseOverDisabled = false;
	    }

	    updateButtons ();

	    showing = false;

	    emit showed ();

	}
	else {

	    updateButtons ();

	    if (new_move.value () < -20000 || new_move.value () > 20000) {

		emit moved ();
		MouseOverDisabled = true;
		emit finished ();

	    }
	    else if (GameRestored) {

		emit moved ();

	    }
	    else {

		emit new_game ();
	    }
	}
    }
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

void
CheckersGameBoard::depthUpButton ()
{
    if (dpth < 16)
	dpth += 2;
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

void
CheckersGameBoard::depthDownButton ()
{
    if (dpth > 4)
	dpth -= 2;
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

void
CheckersGameBoard::newGameButton (bool free_next)
{
    CheckersState *p;
    int i;

    MouseOverDisabled = false;
    GameRestored = false;
    setFirstMove (true);
    setValue (0);
    setState (Init);
    setDefaultPiece (BlankSquare);

    for (i = 0; i < 12; ++i)
	move.square[i] = PlayerPiece;
    for (i = 12; i < 20; ++i)
	move.square[i] = BlankSquare;
    for (i = 20; i < 32; ++i)
	move.square[i] = ComputerPiece;

    move.turn = PlayerPiece;
    move.val = 0;
    move.tdepth = depth ();
    move.jump_start_square = 32;
    move.jump_found = false;
    move.game_over = false;

    move.copy_state (&new_move);
    move.copy_state (&old_move);
    move.copy_state (&start);

    while (HeadPrevStates != NULL) {
	p = HeadPrevStates;
	HeadPrevStates = HeadPrevStates->memPtr;
	delete (p);
    }

    if (free_next) {
	while (HeadNextStates != NULL) {
	    p = HeadNextStates;
	    HeadNextStates = HeadNextStates->memPtr;
	    delete (p);
	}
    }

    PushState (&HeadPrevStates, &start);
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

void
CheckersGameBoard::moveMouse (int i, bool mouse_in_square)
{
    if (mouse_in_square)
	set_mouse_square (i);

    if (MouseOverDisabled)
	return;

    if (!mouse_in_square) {
	new_move.RemoveStateFlags ();
	updateButtons ();
    }
    else {
	setBoardOptions (i);
	if (r->num > 0)
	    updateButtons ();
    }
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

void
CheckersGameBoard::setBoardOptions (int i)
{
    int j, k;

    new_move.RemoveStateFlags ();

    if (i < 32) {

	switch (new_move.square[i]) {

	case PlayerPiece:
	case PlayerKing:

	    new_move.piece = i;

	    if (r != NULL)
		r->free ();
	    r = new CheckersSucessor;
	    r->set_single_step (&new_move);

	    if (r->num > 0) {
		for (j = 0; j < (int) r->num; ++j)
		    if ((k = r->ptr[j]->piece) > 127)
			new_move.square[k - 128] = HiLightedMidSquare;
		    else
			new_move.square[k] = HiLightedEndSquare;

		if (new_move.square[i] == PlayerPiece)
		    new_move.square[i] = HiLightedPlayerPiece;
		else if (new_move.square[i] == PlayerKing)
		    new_move.square[i] = HiLightedPlayerKing;
	    }
	}
    }
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

void
CheckersGameBoard::buttonClicked ()
{
    int i, j;

    i = buttons->findRef ((CheckersButton *) sender ());
    CheckersButton *b = buttons->at (i);

    i = b->pos ();

    switch (b->type ()) {
    case PlayerPiece:
    case PlayerKing:
	setBoardOptions (i);
	if (r->num == 0)
	    break;

    case HiLightedPlayerPiece:
    case HiLightedPlayerKing:
	MouseOverDisabled = true;
	setDefaultPiece (new_move.square[i]);
	new_move.square[i] = HiLightedBlankSquare;
	updateButtons ();
	break;

    case HiLightedMidSquare:
	setDefaultPiece (BlankSquare);
	for (j = 0; j < (int) r->num; ++j) {
	    if (r->ptr[j]->piece - 128 == i) {
		r->ptr[j]->copy_state (&new_move);
		break;
	    }
	}
	setBoardOptions (i);
	updateButtons ();
	break;

    case HiLightedEndSquare:
	setDefaultPiece (BlankSquare);
	for (j = 0; j < (int) r->num; ++j) {
	    if (r->ptr[j]->piece == i) {
		r->ptr[j]->copy_state (&new_move);
		break;
	    }
	}
	updateButtons ();
	makeMove ();
	MouseOverDisabled = false;
	break;

    case HiLightedBlankSquare:
	new_move.square[i] = defaultPiece ();
	MouseOverDisabled = false;
	compulsory_jump ();	// resets MouseOverDisabled on compulsory jump
	setDefaultPiece (BlankSquare);
	updateButtons ();
	break;
    }
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

void
CheckersGameBoard::updateButtons ()
{
    int i;

    if (!inverted ()) {
	for (i = 0; i < 8; i = i + 2) {
	    buttons->at (i)->setPos (3 - ((i / 2) % 4));
	    buttons->at (i)->setType (new_move.square[3 - ((i / 2) % 4)]);
	}
	for (i = 8; i < 16; i = i + 2) {
	    buttons->at (i + 1)->setPos (7 - ((i / 2) % 4));
	    buttons->at (i + 1)->setType (new_move.square[7 - ((i / 2) % 4)]);
	}
	for (i = 16; i < 24; i = i + 2) {
	    buttons->at (i)->setPos (11 - ((i / 2) % 4));
	    buttons->at (i)->setType (new_move.square[11 - ((i / 2) % 4)]);
	}
	for (i = 24; i < 32; i = i + 2) {
	    buttons->at (i + 1)->setPos (15 - ((i / 2) % 4));
	    buttons->at (i +
			 1)->setType (new_move.square[15 - ((i / 2) % 4)]);
	}
	for (i = 32; i < 40; i = i + 2) {
	    buttons->at (i)->setPos (19 - ((i / 2) % 4));
	    buttons->at (i)->setType (new_move.square[19 - ((i / 2) % 4)]);
	}
	for (i = 40; i < 48; i = i + 2) {
	    buttons->at (i + 1)->setPos (23 - ((i / 2) % 4));
	    buttons->at (i +
			 1)->setType (new_move.square[23 - ((i / 2) % 4)]);
	}
	for (i = 48; i < 56; i = i + 2) {
	    buttons->at (i)->setPos (27 - ((i / 2) % 4));
	    buttons->at (i)->setType (new_move.square[27 - ((i / 2) % 4)]);
	}
	for (i = 56; i < 64; i = i + 2) {
	    buttons->at (i + 1)->setPos (31 - ((i / 2) % 4));
	    buttons->at (i +
			 1)->setType (new_move.square[31 - ((i / 2) % 4)]);
	}
    }
    else {
	for (i = 0; i < 8; i = i + 2) {
	    buttons->at (i)->setPos (31 - (3 - ((i / 2) % 4)));
	    buttons->at (i)->setType (new_move.
				      square[31 - (3 - ((i / 2) % 4))]);
	}
	for (i = 8; i < 16; i = i + 2) {
	    buttons->at (i + 1)->setPos (31 - (7 - ((i / 2) % 4)));
	    buttons->at (i +
			 1)->setType (new_move.square[31 -
						      (7 - ((i / 2) % 4))]);
	}
	for (i = 16; i < 24; i = i + 2) {
	    buttons->at (i)->setPos (31 - (11 - ((i / 2) % 4)));
	    buttons->at (i)->setType (new_move.
				      square[31 - (11 - ((i / 2) % 4))]);
	}
	for (i = 24; i < 32; i = i + 2) {
	    buttons->at (i + 1)->setPos (31 - (15 - ((i / 2) % 4)));
	    buttons->at (i +
			 1)->setType (new_move.square[31 -
						      (15 - ((i / 2) % 4))]);
	}
	for (i = 32; i < 40; i = i + 2) {
	    buttons->at (i)->setPos (31 - (19 - ((i / 2) % 4)));
	    buttons->at (i)->setType (new_move.
				      square[31 - (19 - ((i / 2) % 4))]);
	}
	for (i = 40; i < 48; i = i + 2) {
	    buttons->at (i + 1)->setPos (31 - (23 - ((i / 2) % 4)));
	    buttons->at (i +
			 1)->setType (new_move.square[31 -
						      (23 - ((i / 2) % 4))]);
	}
	for (i = 48; i < 56; i = i + 2) {
	    buttons->at (i)->setPos (31 - (27 - ((i / 2) % 4)));
	    buttons->at (i)->setType (new_move.
				      square[31 - (27 - ((i / 2) % 4))]);
	}
	for (i = 56; i < 64; i = i + 2) {
	    buttons->at (i + 1)->setPos (31 - (31 - ((i / 2) % 4)));
	    buttons->at (i +
			 1)->setType (new_move.square[31 -
						      (31 - ((i / 2) % 4))]);
	}
    }
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

void
CheckersGameBoard::computerMove ()
{
    CheckersState *p;
    CheckersSucessor *q;
    CheckersAlternatives *c;

    int i;

    setFirstMove (false);

    while (HeadNextStates != NULL) {
	p = HeadNextStates;
	HeadNextStates = HeadNextStates->memPtr;
	delete (p);
    }

    PushState (&HeadPrevStates, &new_move);

    new_move.RemoveStateFlags ();
    new_move.turn = ComputerPiece;
    new_move.game_over = false;
    new_move.piece = -new_move.value ();
    new_move.copy_state (&move);
    new_move.copy_state (&old_move);

    if (move.piece > 20000) {

	new_move.game_over = true;
	setState (HumanWon);
	setValue (25000);
	MouseOverDisabled = true;
	emit finished ();

    }
    else {

	if (value () < -20000)
	    setTempDepth (temp_depth () - 2);
	else
	    setTempDepth (depth ());

	c = new CheckersAlternatives (&move, temp_depth (), this);
	q = c->alternative_moves ();

	if (q->num > 0) {
	    q->ptr[rand () % q->num]->copy_state (&new_move);
	    if (temp_depth () == depth ())
		setValue (new_move.piece);
	}
	else {
	    if (!StalemateLegal) {
		setState (HumanWon);
		setValue (25000);
	    }
	    else
		setState (NobodyWon);
	    new_move.game_over = true;
	    MouseOverDisabled = true;
	    emit finished ();
	}

	q->free ();
	delete c;
	if (new_move.piece > 20000) {
	    setTempDepth (2);

	    c = new CheckersAlternatives (&move, temp_depth (), this);
	    q = c->alternative_moves ();

	    if (q->num > 0)
		q->ptr[rand () % q->num]->copy_state (&new_move);

	    q->free ();
	    delete c;
	}
	if (value () > -20000)
	    setTempDepth (depth ());

	for (i = 0; i < 64; ++i) {
	    buttons->at (i)->setMouseMoves (0);
	}

	new_move.copy_state (&move);

    }
    if (value () > -20000 && value () < 20000) {
	q = new CheckersSucessor;
	q->set (&move);
	if (q->num == 0) {
	    new_move.game_over = true;
	    setState (NobodyWon);
	    MouseOverDisabled = true;
	    emit finished ();
	}
	q->free ();
    }
    else if (value () < -20000) {
	q = new CheckersSucessor;
	q->set (&new_move);
	if (q->num == 0) {
	    new_move.game_over = true;
	    setState (ComputerWon);
	    setValue (-25000);
	    MouseOverDisabled = true;
	    emit finished ();
	}
	q->free ();
    }

    q = new CheckersSucessor;
    q->set (&new_move);		// Sets Global Variable jump_found
    q->free ();

    player_jump_found = jump_found;

    new_move.jump_found = jump_found;

    compulsory_jump ();

    new_move.jump_start_square = compulsory_jump_start_square;

    new_move.val = value ();

    new_move.tdepth = temp_depth ();

    if (!compulsory_jump_found) {

	for (i = 0; i < 32; ++i) {

	    if (old_move.square[i] != new_move.square[i]) {

		if (old_move.square[i] == ComputerPiece ||
		    old_move.square[i] == ComputerKing)

		    new_move.square[i] = ComputerStartSquare;

		else if (old_move.square[i] == PlayerPiece ||
			 old_move.square[i] == PlayerKing)

		    new_move.square[i] = ComputerJumpedSquare;

		else if (old_move.square[i] == BlankSquare) {

		    if (new_move.square[i] == ComputerPiece)
			new_move.square[i] = ComputerEndSquarePiece;
		    else if (new_move.square[i] == ComputerKing)
			new_move.square[i] = ComputerEndSquareKing;

		}
	    }
	}
    }

    new_move.copy_state (&move);
    PushState (&HeadPrevStates, &new_move);
    updateButtons ();
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

void
CheckersGameBoard::compulsory_jump ()
{
    int i;

    compulsory_jump_found = false;
    compulsory_jump_start_square = 32;

    if (player_jump_found) {	// more than one jump, could still
	// be compulsory if multi-jump.
	for (i = 0; i < 32; ++i) {

	    if (r != NULL) {
		r->free ();
		r = NULL;
	    }

	    setBoardOptions (i);	// Sets Global Variable r

	    if (r != NULL) {

		if (r->num != 0) {
		    if (!compulsory_jump_found) {
			compulsory_jump_found = true;
			compulsory_jump_start_square = i;
		    }
		    else {
			compulsory_jump_found = false;
			compulsory_jump_start_square = 32;
			new_move.RemoveStateFlags ();
			break;
		    }
		}
		r->free ();
		r = NULL;
	    }
	}

	if (compulsory_jump_found) {
	    MouseOverDisabled = true;
	    setBoardOptions (compulsory_jump_start_square);	// Sets Global Variable r       
	}
    }
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

void
CheckersGameBoard::PushState (CheckersState ** r, CheckersState * q)
{
    CheckersState *p;

    p = new CheckersState;
    q->copy_state (p);
    p->RemoveStateFlags ();
    p->memPtr = *r;
    *r = p;
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

void
CheckersGameBoard::Redo ()
{
    CheckersState *p;
    CheckersState *p2;

    p = HeadNextStates;
    if (p == NULL) {
	if (first_move ()) {
	    newGameButton (false);
	    emit new_game ();
	}
	else
	    emit moved ();
	return;
    }
    setFirstMove (false);
    p->copy_state (&new_move);
    p->copy_state (&move);
    p->copy_state (&old_move);
    HeadNextStates = p->memPtr;
    if (HeadNextStates == NULL) {
	PushState (&HeadNextStates, &new_move);
	HeadNextStates->turn = PlayerPiece;
    }
    p2 = HeadPrevStates;
    HeadPrevStates = p;
    HeadPrevStates->memPtr = p2;
    updateButtons ();
    p = HeadNextStates;
    p->copy_state (&new_move);
    HeadNextStates = p->memPtr;
    p2 = HeadPrevStates;
    HeadPrevStates = p;
    HeadPrevStates->memPtr = p2;
    new_move.copy_state (&old_move);
    new_move.copy_state (&move);
    Post_Undo_Redo ();

    MoveTimer = startTimer (1000);
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

void
CheckersGameBoard::Undo ()
{
    CheckersState *p;
    CheckersState *p2;


    p = HeadPrevStates;
    if (p != NULL && p->states_equal (&start)) {
	if (GameRestored) {
	    emit moved ();
	}
	else {
	    emit new_game ();
	}
	return;
    }
    if (p == NULL || p->memPtr == NULL || p->memPtr->memPtr == NULL) {
	return;
    }
    p->copy_state (&new_move);
    HeadPrevStates = p->memPtr;
    if (HeadNextStates == NULL) {
	p2 = HeadNextStates;
	HeadNextStates = p;
	HeadNextStates->memPtr = p2;
    }
    else if (!HeadNextStates->states_equal (&new_move)) {
	p2 = HeadNextStates;
	HeadNextStates = p;
	HeadNextStates->memPtr = p2;
    }
    else
	delete p;

    p = HeadPrevStates;
    p->copy_state (&new_move);
    HeadPrevStates = p->memPtr;
    if (HeadNextStates == NULL) {
	p2 = HeadNextStates;
	HeadNextStates = p;
	HeadNextStates->memPtr = p2;
    }
    else if (!HeadNextStates->states_equal (&new_move)) {
	p2 = HeadNextStates;
	HeadNextStates = p;
	HeadNextStates->memPtr = p2;
    }
    else
	delete p;

    if (start.states_equal (&new_move)) {
	updateButtons ();
	if (GameRestored) {
	    emit moved ();
	}
	else {
	    emit new_game ();
	}
	new_move.copy_state (&old_move);
	new_move.copy_state (&move);
	Post_Undo_Redo ();
	return;
    }
    updateButtons ();

    p = HeadPrevStates;
    p->copy_state (&new_move);
    if (start.states_equal (&new_move)) {
	if (GameRestored) {
	    emit moved ();
	}
	else {
	    emit new_game ();
	}
	new_move.copy_state (&old_move);
	new_move.copy_state (&move);
	Post_Undo_Redo ();

	MoveTimer = startTimer (1000);
	return;
    }

    new_move.copy_state (&old_move);
    new_move.copy_state (&move);

    Post_Undo_Redo ();

    MoveTimer = startTimer (1000);
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

void
CheckersGameBoard::Post_Undo_Redo ()
{
    setDefaultPiece (BlankSquare);
    player_jump_found = new_move.jump_found;
    setValue (new_move.val);
    setTempDepth (new_move.tdepth);
    if (new_move.jump_start_square != 32) {
	setBoardOptions (new_move.jump_start_square);	// Resets Global Variable r
	compulsory_jump_found = true;
	MouseOverDisabled = true;
    }
    else {
	compulsory_jump_found = false;
	MouseOverDisabled = false;
    }
    if (new_move.game_over) {
	MouseOverDisabled = true;
	emit finished ();
    }
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

Checkers::Checkers (QWidget * parent, const char *name)
    :
QWidget (parent, name)
{
    setPalette (black);
    GameOver = false;
    QVBoxLayout *l = new QVBoxLayout (this, 1);

    multiButton = new CheckersGameButton ("", this);
    multiButton->setFixedHeight (BUTTON_HEIGHT);

    connect (multiButton, SIGNAL (clicked ()), SLOT (multiButtonClicked ()));

    board = new CheckersGameBoard (this);
    l->addWidget (board);
    connect (board, SIGNAL (finished ()), SLOT (gameOver ()));
    connect (board, SIGNAL (showed ()), SLOT (doneShowingMove ()));
    connect (board, SIGNAL (moved ()), SLOT (doneMoving ()));
    connect (board, SIGNAL (moving ()), SLOT (Moving ()));
    connect (board, SIGNAL (new_game ()), SLOT (NewGame ()));

    showMoveButton = new CheckersGameButton ("Show Move", this);
    showMoveButton->setFixedHeight (BUTTON_HEIGHT);
    connect (showMoveButton, SIGNAL (clicked ()),
	     SLOT (showMoveButtonClicked ()));

    QHBoxLayout *box1 = new QHBoxLayout (1);
    l->addLayout (box1);

    QVBoxLayout *box2 = new QVBoxLayout (1);
    box1->addLayout (box2);
    box2->addWidget (multiButton);
    box2->addWidget (showMoveButton);

    depthUpButton = new CheckersGameButton ("Depth Up", this);
    depthUpButton->setFixedHeight (BUTTON_HEIGHT);
    connect (depthUpButton, SIGNAL (clicked ()),
	     SLOT (depthUpButtonClicked ()));

    depthDownButton = new CheckersGameButton ("Depth Down", this);
    depthDownButton->setFixedHeight (BUTTON_HEIGHT);
    connect (depthDownButton, SIGNAL (clicked ()),
	     SLOT (depthDownButtonClicked ()));

    QVBoxLayout *box3 = new QVBoxLayout (1);
    box1->addLayout (box3);
    box3->addWidget (depthUpButton);
    box3->addWidget (depthDownButton);

    message = new CheckersGameButton ("", this);
    message->setFixedHeight (BUTTON_HEIGHT * 2 + 1);
    connect (message, SIGNAL (clicked ()), SLOT (about ()));
    box1->addWidget (message);

    undoButton = new CheckersGameButton ("Undo", this);
    undoButton->setFixedHeight (BUTTON_HEIGHT);
    connect (undoButton, SIGNAL (clicked ()), SLOT (undoButtonClicked ()));

    redoButton = new CheckersGameButton ("Redo", this);
    redoButton->setFixedHeight (BUTTON_HEIGHT);
    connect (redoButton, SIGNAL (clicked ()), SLOT (redoButtonClicked ()));

    invertButton = new CheckersGameButton ("Invert", this);
    invertButton->setFixedHeight (BUTTON_HEIGHT);
    connect (invertButton, SIGNAL (clicked ()),
	     SLOT (invertButtonClicked ()));

    QHBoxLayout *box4 = new QHBoxLayout (1);
    l->addLayout (box4);
    box4->addWidget (undoButton);
    box4->addWidget (redoButton);
    box4->addWidget (invertButton);

    newState ();
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

Checkers::~Checkers ()
{
    char buf[80];
    int i;

    QFile f (".checkers");

    if (!f.open (IO_WriteOnly))
	return;

    QRect r = qApp->mainWidget ()->geometry ();

    sprintf (buf,
	     "%d %d %d %d %d %d\n",
	     board->depth (),
	     r.left (), r.top (), r.width (), r.height (),
	     (int) qApp->mainWidget ()->isMaximized ()
	);

    for (i = 0; buf[i] != '\0'; ++i)
	f.putch (buf[i]);

    f.close ();

}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

void
Checkers::doneMoving ()
{
    multiButton->setText ("New Game");

    if (board->state () != HumanWon &&
	board->state () != ComputerWon && board->state () != NobodyWon) {

	board->setState (HumansTurn);
	newState ();
    }
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

void
Checkers::NewGame ()
{
    multiButton->setText ("Play");
    board->setState (Init);
    newState ();
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

void
Checkers::Moving ()
{
    if (board->state () != HumanWon &&
	board->state () != ComputerWon && board->state () != NobodyWon)

	board->setState (Thinking);

    newState ();
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

void
Checkers::showMoveButtonClicked ()
{
    if (board->state () != HumanWon &&
	board->state () != ComputerWon &&
	board->state () != NobodyWon && board->state () != Init)

	board->setState (Showing);

    newState ();

    board->showMoveButton ();
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

void
Checkers::doneShowingMove ()
{
    if (board->state () != HumanWon &&
	board->state () != ComputerWon &&
	board->state () != NobodyWon && board->state () != Init)

	board->setState (HumansTurn);

    newState ();
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

void
Checkers::depthUpButtonClicked ()
{
    board->depthUpButton ();
    newState ();
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

void
Checkers::depthDownButtonClicked ()
{
    board->depthDownButton ();
    newState ();
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

void
Checkers::multiButtonClicked ()
{
    if (board->state () == Init) {

	board->setState (Thinking);
	newState ();
	board->makeMove ();

    }
    else if (board->state () == Thinking) {

	board->setState (Aborting);
	newState ();

    }
    else if (board->state () == Aborting) {


    }
    else if (!GameOver) {
	switch (QMessageBox::warning
		(this, "Qt Checkers", "\n    Start New Game ?    \n", "&Yes",
		 "&No", 0, 0, 1)) {
	case 0:
	    board->setState (Init);
	    board->newGameButton (true);
	    board->updateButtons ();
	    newState ();
	    multiButton->setText ("Play");
	    break;
	case 1:
	    break;
	}
    }
    else {
	GameOver = false;
	board->setState (Init);
	board->newGameButton (true);
	board->updateButtons ();
	newState ();
	multiButton->setText ("Play");
    }
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

void
Checkers::gameOver ()
{
    GameOver = true;
    newState ();
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

void
Checkers::invertButtonClicked ()
{
    board->setInverted (!board->inverted ());
    board->updateButtons ();
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

void
Checkers::undoButtonClicked ()
{
    if (board->state () == Undoing)
	return;
    GameOver = false;
    board->setState (Undoing);
    QTimer::singleShot (50, board, SLOT (Undo ()));
    newState ();
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

void
Checkers::redoButtonClicked ()
{
    if (board->state () == Redoing)
	return;
    board->setState (Redoing);
    QTimer::singleShot (50, board, SLOT (Redo ()));
    newState ();
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

void
Checkers::about ()
{
    QMessageBox::about (this,
			"Qt Checkers",
			"\n\n"
			"      (C)  David A.S.Crofts                         \n\n"
			"      December 30, 2013\n\n"
			"      23 Brisbane Street\n"
			"      Berwick Victoria 3806\n"
			"      Australia\n\n"
			"      dasc1961@netscape.net\n\n");
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

void
Checkers::newState ()
{
    int v;
    QString work, num1, num2;

    QString msg[] = {
	"Depth = ",
	"Depth = ",
	"You Win !!!",
	"I Win !!!",
	"Draw !!!",
	"** SHOWING **",
	"** MOVING **",
	"** UNDO **",
	"** REDO **",
	"** ABORT **"
    };

    if (board->state () == 0) {

	num1.setNum (board->depth ());
	num2.setNum (board->value ());
	work = msg[0] + num1 + "\n\nValue(" + num2 + ")";

    }
    else if (board->state () == 1) {

	if ((v = board->value ()) < -20000)
	    work = msg[3];
	else if (v > 20000)
	    work = msg[2];
	else {
	    num1.setNum (v);
	    num2.setNum (board->depth ());
	    work = msg[1] + num2 + "\n\nValue(" + num1 + ")";
	}

    }
    else

	work = msg[board->state ()];

    message->setText (work);
}

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------
