//--------------------------------------------------------------------------------
//
// Checkers	
// ========
//
// by:-	David A.S.Crofts
//
// 23 Brisbane Street 
// Berwick Victoria 3806
// Australia
//
// dasc1961@netscape.net
//
// http://www.davidcrofts.com.au/
//
//--------------------------------------------------------------------------------

#ifndef CHECKERS_H
#define CHECKERS_H

#include <qpushbutton.h>
#include <qwidgetstack.h>
#include <qvector.h>
#include <qlayout.h>
#include <qapplication.h>
#include <qlabel.h>
#include <qtimer.h>
#include <qmessagebox.h>
#include <qwidgetlist.h>

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

#define  BUTTON_HEIGHT		  40		// Default Checkers Game Button Height.
#define  StalemateLegal	       false		// When "false" you lose when cant move.
						// When "true" stalemates are considered a draw.
//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

#define  PlayerPiece		   0		// Used to specify piece type in board.
#define  PlayerKing		   1		// Also used as indexes to moves[] etc.
#define  ComputerPiece		   2
#define  ComputerKing		   3

#define  BlankSquare               4

#define  HighestValidPiece         4

#define  HiLightedPlayerPiece      5
#define  HiLightedPlayerKing       6
#define  HiLightedBlankSquare      7
#define  HiLightedEndSquare        8
#define  HiLightedMidSquare        9

#define  ComputerStartSquare      10
#define  ComputerJumpedSquare     11
#define  ComputerEndSquarePiece   12
#define  ComputerEndSquareKing    13

#define  GreenSquare              14

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

enum State 
{ 
    Init, 
    HumansTurn, 
    HumanWon, 
    ComputerWon, 
    NobodyWon, 
    Showing, 
    Thinking, 
    Undoing, 
    Redoing, 
    Aborting 
};

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

class CheckersPieceBlock
{
  public:

    CheckersPieceBlock (unsigned char mp, unsigned char mk, unsigned char op, unsigned char ok);

    unsigned char mover_piece;
    unsigned char mover_king;
    unsigned char opponent_piece;
    unsigned char opponent_king;
};

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

class CheckersState
{
  public:
    
    void copy_state (CheckersState *);
    void copy_state_play (CheckersState *);
    void RemoveStateFlags ();
    void set_clump_value ();
    bool states_equal (CheckersState *);
    int  value ();

    unsigned char square[32];
    unsigned char turn;

    long piece;
    long clump;
												
    int	jump_start_square;		
    int	jump_found;			
    int	val;
    int	tdepth;
    int	game_over;

    CheckersState * memPtr;
};

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

class CheckersSucessor
{
  public:
    
    CheckersSucessor ();
	  
    void set (CheckersState *);
    void set_single_step (CheckersState *);
    void end_game_tweek ();
    void free ();
    void add_jumps (CheckersState *, bool);
    
    CheckersSucessor *consider_jumps (CheckersState *);

    unsigned int num;
    CheckersState * ptr[32];
};

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

class CheckersButton : public QPushButton
{
  Q_OBJECT
  
  public:
  
    CheckersButton (QWidget * parent);

    int			mouse_moves () const 				{ return m; }
    void 		setMouseMoves (int mm) 				{ m = mm; }
	
    unsigned char 	pos () const 					{ return p; }
    void 		setPos (unsigned char pos) 			{ p = pos; }
    
    unsigned char 	type () const					{ return t; }
    void 		setTypeNoPaint (unsigned char tt)		{ t = tt; }
	
    void setType (unsigned char type)
    {
	if (t != type) {
	    t = type;
	    repaint ();
	}
    }

    QSizePolicy sizePolicy () const
    {
	return QSizePolicy (QSizePolicy::Expanding, QSizePolicy::Expanding);
    }

  signals:
	
    void mouse (int, bool);
  
  protected:
	
    void drawButtonLabel (QPainter *);
    void enterEvent (QEvent *);
    void leaveEvent (QEvent *);
    void mouseMoveEvent (QMouseEvent *);

  private:
	  
    unsigned char t;
    unsigned char p;
    int m;
};

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

typedef QVector < CheckersButton > CheckersButtons;

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

class CheckersGameButton : public QPushButton
{
  Q_OBJECT

  public:

    CheckersGameButton (const QString & text, QWidget * parent, const char * name = 0);

    bool mouse_over () const 		{ return m; }
    void setMouseOver (bool mm) 	{ m = mm; }
	
  protected:

    void drawButtonLabel (QPainter *);
    void enterEvent (QEvent *);
    void leaveEvent (QEvent *);
	
  private:
	  
    bool m;
};

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

class CheckersAlternatives : public QWidget
{
  Q_OBJECT
  
  public:
    
    CheckersAlternatives (CheckersState * p, unsigned int depth, QWidget * parent = 0, const char * name = 0);

    CheckersSucessor * 	alternative_moves () const 					{ return a; }
    void		setAlternatives (CheckersSucessor * aa) 			{ a = aa; }
	
    int			f (CheckersState *, int, int, unsigned int);
	
  private:
	  
    CheckersSucessor * a;
};

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

class CheckersLabel : public QLabel
{
  Q_OBJECT
  
  public:
    
    CheckersLabel (QWidget * parent, const char * name = 0, WFlags f = 0);

    QSizePolicy sizePolicy () const
    {
	return QSizePolicy (QSizePolicy::Fixed, QSizePolicy::Fixed);
    }

    QSize sizeHint () const
    {
	int h = BUTTON_HEIGHT + 1;
	int win_width = qApp->mainWidget()->width();
	int win_height = qApp->mainWidget()->height();
		
	int square_width = win_width - 2 * h;
	int square_height = win_height - 5 * h;
		
	if (square_width > square_height)
            return QSize ((win_width - square_height) / 2, h);	    
	else
	    return QSize (h, (win_height - square_width - 3 * h) / 2);
    }
};

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

class CheckersGrid : public QWidget
{
  Q_OBJECT
  
  public:
  
    CheckersGrid (QWidget * parent = 0, const char * name = 0);

    CheckersButtons * checkers_buttons () const 			   { return b; }
    void 	      setCheckersButtons (CheckersButtons *bb) 		   { b = bb; }
	
  private:
	  
    CheckersButtons * b;
};

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

class CheckersGameBoard : public QWidget
{
  Q_OBJECT
  
  public:
  
    CheckersGameBoard (QWidget * parent = 0, const char * name = 0);
    ~CheckersGameBoard ();

    bool first_move () const 				{ return firstm; }
    void setFirstMove (bool fm) 			{ firstm = fm; }
    
    bool inverted () const 				{ return inv; }
    void setInverted (bool b) 				{ inv = b; }
    
    unsigned char defaultPiece () const 		{ return defp; }
    void setDefaultPiece (unsigned char dp) 		{ defp = dp; }
	
    int depth () const 					{ return dpth; }
    void setDepth (int d) 				{ dpth = d; }
    
    int temp_depth () const 				{ return tdpth; }
    void setTempDepth (int td) 				{ tdpth = td; }
    
    int value () const 					{ return val; }
    void setValue (int v) 				{ val = v; }
    
    int mouse_square () const 				{ return ms; }
    void set_mouse_square (int m) 			{ ms = m; }
    
    State state () const 				{ return st; }
    void setState (State state) 			{ st = state; }
    
    CheckersState * move_ptr ()  			{ return & move; }
    
    void makeMove ();
    void setBoardOptions (int);
    void showMoveButton ();
    void depthUpButton ();
    void depthDownButton ();
    void newGameButton (bool);
    void compulsory_jump ();
    void updateButtons ();
    void Post_Undo_Redo ();
	
    void timerEvent (QTimerEvent *);
	
  signals:
  
    void finished ();
    void showed ();
    void moving ();
    void new_game ();
    void moved ();

  private slots:
  
    void buttonClicked ();
    void moveMouse (int, bool);
    void Undo ();
    void Redo ();

  private:
  
    void showMove ();
    void computerMove ();
    void PushState (CheckersState **, CheckersState *);

    int MoveTimer;
    bool firstm;
    int inv;
    unsigned char defp;
    int dpth;
    int tdpth;
    int val;
    int ms;
    State st;
    int MouseOverDisabled;
    bool GameRestored;
    int compulsory_jump_found;
    int compulsory_jump_start_square;
    bool showing;

    CheckersSucessor * r;

    CheckersState * HeadPrevStates;
    CheckersState * HeadNextStates;

    CheckersState move;
    CheckersState new_move;
    CheckersState old_move;
    CheckersState start;

    CheckersGrid * grid;
	
    CheckersButtons * buttons;
};

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

class Checkers : public QWidget
{
  Q_OBJECT
  
  public:
  
    Checkers (QWidget * parent = 0, const char * name = 0);
    ~Checkers ();
	
  private slots:
  
    void multiButtonClicked ();
    void showMoveButtonClicked ();
    void depthUpButtonClicked ();
    void depthDownButtonClicked ();
    void invertButtonClicked ();
    void undoButtonClicked ();
    void redoButtonClicked ();
    void about ();
    void gameOver ();
    void doneShowingMove ();
    void Moving ();
    void NewGame ();
    void doneMoving ();

  private:
  
    void newState ();

    CheckersGameBoard  * board;
    
    CheckersGameButton * multiButton;
    CheckersGameButton * showMoveButton;
    CheckersGameButton * depthUpButton;
    CheckersGameButton * depthDownButton;
    CheckersGameButton * invertButton;
    CheckersGameButton * undoButton;
    CheckersGameButton * redoButton;
    CheckersGameButton * message;

    bool GameOver;

  protected:

    void resizeEvent (QResizeEvent *)
    {
	QWidgetList * list = QApplication::allWidgets ();
	QWidgetListIt it( *list );         
	QWidget * w;
	while ((w = it.current()) != NULL) {
	    ++it;
            w->updateGeometry ();
	}
	delete list;   
    }
};

//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------

#endif // CHECKERS_H

